"""Test fixtures for JoyfulJay extractors."""

from .packets import (
    create_connection_flows,
    create_dns_flow,
    create_dns_query,
    create_dns_response,
    create_entropy_flow,
    create_fingerprint_flow,
    create_padding_flow,
    create_quic_flow,
    create_quic_initial_packet,
    create_quic_short_header_packet,
    create_ssh_flow,
    create_ssh_kex_init,
    create_ssh_version_exchange,
    create_tls_certificate,
    create_tls_client_hello,
    create_tls_flow,
    create_tls_server_hello,
)

from .tranalyzer_packets import (
    # MAC fixtures
    create_broadcast_mac_packet,
    create_mac_flow,
    create_mac_packet,
    create_multicast_mac_packet,
    create_vlan_tagged_packet,
    # IP Extended fixtures
    create_ip_extended_packet,
    create_ip_flow_with_dscp,
    create_ip_flow_with_ttl_variations,
    # IPv6 fixtures
    create_ipv6_flow,
    create_ipv6_packet,
    # TCP Sequence fixtures
    create_tcp_dup_ack_flow,
    create_tcp_handshake_flow,
    create_tcp_retransmission_flow,
    create_tcp_seq_packet,
    # TCP Window fixtures
    create_tcp_window_flow,
    create_tcp_window_packet,
    create_zero_window_flow,
    # TCP Options fixtures
    create_tcp_options_flow,
    create_tcp_options_packet,
    create_tcp_sack_flow,
    # MPTCP fixtures
    create_mptcp_capable_option,
    create_mptcp_flow,
    create_non_mptcp_flow,
    # TCP RTT fixtures
    create_tcp_rtt_flow,
    # TCP Fingerprint fixtures
    create_linux_fingerprint_flow,
    create_tcp_fingerprint_flow,
    create_windows_fingerprint_flow,
    # ICMP fixtures
    create_icmp_packet,
    create_icmp_ttl_exceeded_flow,
    create_icmp_unreachable_flow,
    create_ping_flow,
    # Combined fixtures
    create_empty_flow,
    create_full_featured_tcp_flow,
    create_non_tcp_flow,
)

__all__ = [
    # Existing fixtures
    "create_connection_flows",
    "create_dns_flow",
    "create_dns_query",
    "create_dns_response",
    "create_entropy_flow",
    "create_fingerprint_flow",
    "create_padding_flow",
    "create_quic_flow",
    "create_quic_initial_packet",
    "create_quic_short_header_packet",
    "create_ssh_flow",
    "create_ssh_kex_init",
    "create_ssh_version_exchange",
    "create_tls_certificate",
    "create_tls_client_hello",
    "create_tls_flow",
    "create_tls_server_hello",
    # Tranalyzer fixtures
    "create_broadcast_mac_packet",
    "create_empty_flow",
    "create_full_featured_tcp_flow",
    "create_icmp_packet",
    "create_icmp_ttl_exceeded_flow",
    "create_icmp_unreachable_flow",
    "create_ip_extended_packet",
    "create_ip_flow_with_dscp",
    "create_ip_flow_with_ttl_variations",
    "create_ipv6_flow",
    "create_ipv6_packet",
    "create_linux_fingerprint_flow",
    "create_mac_flow",
    "create_mac_packet",
    "create_mptcp_capable_option",
    "create_mptcp_flow",
    "create_multicast_mac_packet",
    "create_non_mptcp_flow",
    "create_non_tcp_flow",
    "create_ping_flow",
    "create_tcp_dup_ack_flow",
    "create_tcp_fingerprint_flow",
    "create_tcp_handshake_flow",
    "create_tcp_options_flow",
    "create_tcp_options_packet",
    "create_tcp_retransmission_flow",
    "create_tcp_rtt_flow",
    "create_tcp_sack_flow",
    "create_tcp_seq_packet",
    "create_tcp_window_flow",
    "create_tcp_window_packet",
    "create_vlan_tagged_packet",
    "create_windows_fingerprint_flow",
    "create_zero_window_flow",
]
